/*
 * file CcElementType.java
 *
 * Licensed Materials - Property of IBM
 * Restricted Materials of IBM 
 *
 * com.ibm.rational.wvcm.stp.cc.CcElementType
 *
 *  Copyright IBM Corporation 2004, 2008.  All Rights Reserved.
 * Note to U.S. Government Users Restricted Rights:  Use, duplication or 
 * disclosure restricted by GSA ADP  Schedule Contract with IBM Corp.
 */
package com.ibm.rational.wvcm.stp.cc;

import static com.ibm.rational.wvcm.stpex.StpExBase.PROPERTY_NAMESPACE;

import javax.wvcm.WvcmException;
import javax.wvcm.PropertyNameList.PropertyName;

import com.ibm.rational.wvcm.stp.cc.CcTypeBase.Scope;
import com.ibm.rational.wvcm.stpex.StpExEnumeration;
import com.ibm.rational.wvcm.stpex.StpExEnumerationBase;

/**
 * <p>
 * A proxy for a ClearCase element type.
 * </p>
 * <p>
 * The use of element types lets each class of elements be handled differently. 
 * An element type is a class of file elements. Predefined element types, such as 
 * <b>file</b> and <b>text_file</b>, are included. You can define your own element types. 
 * When you create an element type for use in UCM projects, you can specify a mergetype 
 * attribute, which determines how deliver and rebase operations handle merging of 
 * files of that element type.
 * </p>
 * <p>
 * For more information about elements and element types, see the ClearCase "Guide to Managing Software Projects"
 * manual, and the cleartool man pages "mkeltype" and "mkelem".
 * </p>
 *
 */
public interface CcElementType extends CcTypeBase
{
    /**
     * Merge type of an element type.<p>
     * The merge type defines the behaviour when a merge is performed 
     * via deliver, rebase or an explicit findmerge command.
     * The four merge types are:
     * <ul>
     * <li> Auto (default): A findmerge operation attempts to merge elements of this type.
     * <li> User: A findmerge operation performs trivial merges only. 
     * Nontrivial merges must be performed manually.
     * <li> Never: A findmerge operation ignores elements of this type. 
     * The never attribute is useful for working with files such as 
     * binary files or bitmap graphics images.
     * <li> Copy: A findmerge operation attempts to merge elements of this type 
     * automatically by copying the from-version to the to-version 
     * (replacing the to-version with the from-version).
     * </ul>
     */
    enum MergeType implements StpExEnumeration {
        
        /** Automatically perform merges */
        AUTO,
        
        /** Automatically perform trivial merges, manual for non-trivial */
        USER,
        
        /** Never merge */
        NEVER,
        
        /** Copy version to merge */
        COPY;
    }
    
    /**
     * The merge type of this element type.
     */
    PropertyName<MergeType> MERGE_TYPE =
        new PropertyName<MergeType>(PROPERTY_NAMESPACE, "merge-type");
    
    /**
     * Get the value of this proxy's {@link #MERGE_TYPE} property.
     * @return this element type's merge type
     * @throws WvcmException 
     */
    MergeType getMergeType() throws WvcmException;

    /**
     * The type manager of this element type.
     */
    PropertyName<String> TYPE_MANAGER =
        new PropertyName<String>(PROPERTY_NAMESPACE, "type-manager");

    
    /**
     * Get the value of this proxy's {@link #TYPE_MANAGER} property.
     * @return this element type's type manager
     * @throws WvcmException 
     */
    String getTypeManager() throws WvcmException;

}
