/*
 * file CqHitSet.java
 *
 * Licensed Materials - Property of IBM
 * Restricted Materials of IBM 
 *
 * com.ibm.rational.wvcm.stp.cq.CqHitSet
 *
 *  Copyright IBM Corporation 2008.  All Rights Reserved.
 * Note to U.S. Government Users Restricted Rights:  Use, duplication or 
 * disclosure restricted by GSA ADP  Schedule Contract with IBM Corp.
 */

package com.ibm.rational.wvcm.stp.cq;

import java.util.Iterator;

import javax.wvcm.WvcmException;

import com.ibm.rational.wvcm.stp.StpReleasable;

/**
 * The interface to the results of a CqUserDb.doFullTextSearch invocation.
 * 
 * A CqHitSet is an Iterable, but each invocation of its iterator() method does
 * not restart the iteration. It just continues from the last read hit. That is
 * CqHitSet.iterator() simply returns the CqHitSet object. It is provided to
 * allow the use of CqHitSet objects in the Java 5 for-each construct.
 * 
 * @see CqUserDb#doFullTextSearch(com.ibm.rational.wvcm.stp.cq.CqUserDb.SearchFilter, long[])
 */
public interface CqHitSet extends StpReleasable, Iterator<CqHit>,
                Iterable<CqHit>
{
    /**
     * @return The total number of hits found by the search. In most cases this
     *         number is only an approximation and can be used only to give an
     *         indication of the magnitude of the hit set.
     */
    long totalHits();

    /**
     * @return The search string composed from the filter content and used by 
     * the search engine to generate this hit set.
     */
    String expandedSearchString();

    /**
     * @return Amongst all hits found by the search, the ordinal position of the
     *         hit to be returned by the next call to Iterator.next(). This
     *         value can exceed totalHits() while there are still hits
     *         available. All hits have been returned only when hasNext() and
     *         hasMore() both return <b>false</b>.
     */
    long nextIndex();

    /**
     * @return The number of hits currently on the client that can be accessed
     *         by Iterator.next(). This value resets after each interaction with
     *         the server and thus is not monotonic.
     */
    long size();

    /**
     * Indicates whether or not the server has more data to provide via the
     * doGetMore() method.
     * 
     * @return <b>true</b> if the server has more hit data that it could send
     *         to the client.
     */
    boolean hasMore();

    /**
     * Requests the server to forward the next set of hits to the client.
     * Successful execution of this method flushes the current content of the
     * CqHitSet iterator and resets it to the first hit of the new set,
     * adjusting the nextIndex and size fields accordingly.
     * 
     * @param maxSetSize The maximum number of hits the client will accept in
     *            the next set. If zero, the number of hits requested will be
     *            the same as previously request.
     * 
     * @return <b>true</b> if more hits were obtained from the server; <b>false</b>
     *         if there were none left on the server.
     * @throws WvcmException If the server had indicated it had more hits, but
     *             the attempt to get those hits from the server failed.
     */
    boolean doGetMore(long maxSetSize) throws WvcmException;

    /**
     * Sets whether or not more hits are automatically requested from the server
     * when those currently on the client run out.
     * 
     * @param autoGetMore If <b>false</b>, Iterator.hasNext() will return
     *            <b>false</b> and Iterator.next() will throw an exception when
     *            size() returns 0 (i.e. after the last hit on the client has
     *            been returned by the Iterator.next() function). This is the
     *            initial, default behavior of any result set.
     *            <p>
     *            If <b>true</b>, Iterator.hasNext() or Iterator.next() will
     *            automatically call doGetMore() when there are no more hits on
     *            the client. In this mode the last specified value for the hit
     *            set size is used in the implicit call of doGetMore().
     */
    void setAutoGetMore(boolean autoGetMore);
}
